@@ -1,6 +1,6 @@
 ;;; helm-imenu.el --- Helm interface for Imenu -*- lexical-binding: t -*-
 
-;; Copyright (C) 2012 ~ 2016 Thierry Volpiatto <thierry.volpiatto@gmail.com>
+;; Copyright (C) 2012 ~ 2017 Thierry Volpiatto <thierry.volpiatto@gmail.com>
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -124,8 +124,9 @@ only '((foo . bar)) is needed."
                  :fuzzy-match helm-imenu-fuzzy-match))))
 
 (defun helm-imenu--maybe-switch-to-buffer (candidate)
-  (helm-aif (marker-buffer (cdr candidate))
-      (switch-to-buffer it)))
+  (let ((cand (cdr candidate)))
+    (helm-aif (and (markerp cand) (marker-buffer cand))
+        (switch-to-buffer it))))
 
 (defun helm-imenu--execute-action-at-once-p ()
   (let ((cur (helm-get-selection))
@@ -191,19 +192,23 @@ only '((foo . bar)) is needed."
 
 (defun helm-imenu--candidates-1 (alist)
   (cl-loop for elm in alist
-           nconc (if (imenu--subalist-p elm)
-                     (helm-imenu--candidates-1
-                      (cl-loop for (e . v) in (cdr elm) collect
-                               (cons (propertize
-                                      e 'helm-imenu-type (car elm))
-                                     ;; If value is an integer, convert it
-                                     ;; to a marker, otherwise it is a cons cell
-                                     ;; and it will be converted on next recursions.
-                                     ;; (Issue #1060) [1]. 
-                                     (if (integerp v) (copy-marker v) v))))
-                     (and (cdr elm) ; bug in imenu, should not be needed.
-                          (setcdr elm (copy-marker (cdr elm))) ; Same as [1].
-                          (list elm)))))
+           nconc (cond
+                  ((imenu--subalist-p elm)
+                   (helm-imenu--candidates-1
+                    (cl-loop for (e . v) in (cdr elm) collect
+                             (cons (propertize
+                                    e 'helm-imenu-type (car elm))
+                                   ;; If value is an integer, convert it
+                                   ;; to a marker, otherwise it is a cons cell
+                                   ;; and it will be converted on next recursions.
+                                   ;; (Issue #1060) [1].
+                                   (if (integerp v) (copy-marker v) v)))))
+                  ((listp (cdr elm))
+                   (and elm (list elm)))
+                  (t
+                   (and (cdr elm) ; bug in imenu, should not be needed.
+                        (setcdr elm (copy-marker (cdr elm))) ; Same as [1].
+                        (list elm))))))
 
 (defun helm-imenu--get-prop (item)
   ;; property value of ITEM can have itself
@@ -222,7 +227,7 @@ only '((foo . bar)) is needed."
   (cl-loop for (k . v) in candidates
         for types = (or (helm-imenu--get-prop k)
                         (list "Function" k))
-        for bufname = (buffer-name (marker-buffer v))
+        for bufname = (and (markerp v) (buffer-name (marker-buffer v)))
         for disp1 = (mapconcat
                      (lambda (x)
                        (propertize
