@@ -1,6 +1,6 @@
 ;;; helm-mode.el --- Enable helm completion everywhere. -*- lexical-binding: t -*-
 
-;; Copyright (C) 2012 ~ 2016 Thierry Volpiatto <thierry.volpiatto@gmail.com>
+;; Copyright (C) 2012 ~ 2017 Thierry Volpiatto <thierry.volpiatto@gmail.com>
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -150,7 +150,7 @@ and all functions belonging in this list from `minibuffer-setup-hook'."
 (defvar helm-cr-unknown-pattern-flag nil)
 
 
-;;; Helm `completing-read' replacement
+;;; helm-comp-read
 ;;
 ;;
 (defun helm-cr-empty-string ()
@@ -309,6 +309,7 @@ If COLLECTION is an `obarray', a TEST should be needed. See `obarray'."
                             help-message
                             (keymap helm-comp-read-map)
                             (name "Helm Completions")
+                            header-name
                             candidates-in-buffer
                             match-part
                             exec-when-only-one
@@ -320,7 +321,9 @@ If COLLECTION is an `obarray', a TEST should be needed. See `obarray'."
                             marked-candidates
                             nomark
                             (alistp t)
-                            (candidate-number-limit helm-candidate-number-limit))
+                            (candidate-number-limit helm-candidate-number-limit)
+                            multiline
+                            allow-nest)
   "Read a string in the minibuffer, with helm completion.
 
 It is helm `completing-read' equivalent.
@@ -378,6 +381,8 @@ Keys description:
 
 - NAME: The name related to this local source.
 
+- HEADER-NAME: A function to alter NAME, see `helm'.
+
 - EXEC-WHEN-ONLY-ONE: Bound `helm-execute-action-at-once-if-one'
   to non--nil. (possibles values are t or nil).
 
@@ -408,6 +413,11 @@ Keys description:
 - MATCH-PART: Allow matching only one part of candidate.
   See match-part documentation in `helm-source'.
 
+- ALLOW-NEST: Allow nesting this `helm-comp-read' in a helm session.
+  See `helm'.
+
+- MULTILINE: See multiline in `helm-source'.
+
 Any prefix args passed during `helm-comp-read' invocation will be recorded
 in `helm-current-prefix-arg', otherwise if prefix args were given before
 `helm-comp-read' invocation, the value of `current-prefix-arg' will be used.
@@ -467,6 +477,7 @@ that use `helm-comp-read' See `helm-M-x' for example."
            (src-hist (helm-build-sync-source (format "%s History" name)
                          :candidates history-get-candidates
                          :fuzzy-match fuzzy
+                         :multiline multiline
                          :match-part match-part
                          :filtered-candidate-transformer
                          (append '((lambda (candidates sources)
@@ -486,6 +497,8 @@ that use `helm-comp-read' See `helm-M-x' for example."
            (src (helm-build-sync-source name
                   :candidates get-candidates
                   :match-part match-part
+                  :multiline multiline
+                  :header-name header-name
                   :filtered-candidate-transformer fc-transformer
                   :requires-pattern requires-pattern
                   :persistent-action persistent-action
@@ -498,6 +511,8 @@ that use `helm-comp-read' See `helm-M-x' for example."
            (src-1 (helm-build-in-buffer-source name
                     :data get-candidates
                     :match-part match-part
+                    :multiline multiline
+                    :header-name header-name
                     :filtered-candidate-transformer fc-transformer
                     :requires-pattern requires-pattern
                     :persistent-action persistent-action
@@ -525,6 +540,7 @@ that use `helm-comp-read' See `helm-M-x' for example."
                          :preselect preselect
                          :prompt prompt
                          :resume 'noresume
+                         :allow-nest allow-nest
                          :candidate-number-limit candidate-number-limit
                          :case-fold-search case-fold
                          :keymap loc-map
@@ -543,6 +559,7 @@ that use `helm-comp-read' See `helm-M-x' for example."
                (set history (list result)))))
       (or result (helm-mode--keyboard-quit)))))
 
+
 ;; Generic completing-read
 ;;
 ;; Support also function as collection.
@@ -854,6 +871,7 @@ Keys description:
              :nohighlight t
              :persistent-action persistent-action
              :persistent-help persistent-help
+             :keymap cmap
              :nomark nomark
              :action action-fn)
            ;; Other source.
@@ -886,6 +904,7 @@ Keys description:
              :persistent-action persistent-action
              :persistent-help persistent-help
              :volatile t
+             :keymap cmap
              :cleanup 'helm-find-files-cleanup
              :nomark nomark
              :action action-fn)))
@@ -1013,6 +1032,28 @@ Don't use it directly, use instead `helm-read-file-name' in your programs."
     (if (eq predicate 'file-directory-p) ; Using `read-directory-name'.
         (file-name-as-directory fname) fname)))
 
+;; Read file name handler with history (issue #1652)
+(defun helm-read-file-name-handler-1 (prompt dir default-filename
+                                      mustmatch initial predicate
+                                      name buffer)
+  "A `read-file-name' handler with history.
+Can be added to `helm-completing-read-handlers-alist' for functions
+that need a `read-file-name' function with directory history.
+The `helm-find-files' history `helm-ff-history' is used here."
+  (helm-read-file-name
+   prompt
+   :name name
+   :history helm-ff-history
+   :buffer buffer
+   :default default-filename
+   :initial-input (expand-file-name initial dir)
+   :alistp nil
+   :must-match mustmatch
+   :test predicate))
+
+
+;;; Completion in region
+;;
 (defun helm-mode--advice-lisp--local-variables (old--fn &rest args)
   (ignore-errors
     (apply old--fn args)))
