@@ -386,6 +386,19 @@
   This attribute is not supported for asynchronous sources
   since they perform pattern matching themselves.")
 
+   (redisplay
+    :initarg :redisplay
+    :initform 'identity
+    :custom (choice list function)
+    :documentation
+    "  A function or a list of functions to apply to current list
+  of candidates when redisplaying buffer with `helm-redisplay-buffer'.
+  This is only interesting for modifying and redisplaying the whole list
+  of candidates in async sources.
+  It uses `identity' by default for when async sources are mixed with
+  normal sources, in this case these normal sources are not modified and
+  redisplayed as they are.")
+
    (nomark
     :initarg :nomark
     :initform nil
@@ -510,7 +523,9 @@ With a value of 1 enable, a value of -1 or nil disable the mode.
   If source contain match-part attribute, match is computed only
   on part of candidate returned by the call of function provided
   by this attribute. The function should have one arg, candidate,
-  and return only a specific part of candidate.")
+  and return only a specific part of candidate.
+  On async sources, as matching is done by the backend, this have
+  no effect apart for highlighting matches.")
 
    (before-init-hook
     :initarg :before-init-hook
