@@ -1,6 +1,6 @@
 ;;; helm-utils.el --- Utilities Functions for helm. -*- lexical-binding: t -*-
 
-;; Copyright (C) 2012 ~ 2016 Thierry Volpiatto <thierry.volpiatto@gmail.com>
+;; Copyright (C) 2012 ~ 2017 Thierry Volpiatto <thierry.volpiatto@gmail.com>
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -666,6 +666,10 @@ Assume regexp is a pcre based regexp."
   (when helm--match-item-overlays
     (mapc 'delete-overlay helm--match-item-overlays)))
 
+(defun helm-match-line-cleanup-maybe ()
+  (when (helm-empty-buffer-p)
+    (helm-match-line-cleanup)))
+
 (defun helm-match-line-update ()
   (when helm-match-line-overlay
     (delete-overlay helm-match-line-overlay)
@@ -680,6 +684,7 @@ Assume regexp is a pcre based regexp."
 (defun helm-match-line-cleanup-pulse ()
   (run-with-timer 0.3 nil #'helm-match-line-cleanup))
 
+(add-hook 'helm-after-update-hook 'helm-match-line-cleanup-maybe)
 (add-hook 'helm-after-persistent-action-hook 'helm-persistent-autoresize-hook)
 (add-hook 'helm-cleanup-hook 'helm-match-line-cleanup)
 (add-hook 'helm-after-action-hook 'helm-match-line-cleanup-pulse)
